#---------------------------------------------------------------------------
#
#   HIIW - Attributes stored outside object
#
#---------------------------------------------------------------------------
#
#   A barnacle is an attribute whose value is kept outside
#   of the object it belongs to. One use for this is to
#   prevent attributes from being pickled.
#

import weakref

dicts = weakref.WeakKeyDictionary()

def get_dict(obj):
	d = dicts.get(obj)
	if d is None:
		d = {}
		dicts[obj] = d
	return d

def barnacle(name, default = None):
	def get(self):
		d = get_dict(self)
		return d.get(name, default)
	def set(self, value):
		d = get_dict(self)
		d[name] = value
	return property(get, set)
