#---------------------------------------------------------------------------
#
#   HIIW - Characters
#
#---------------------------------------------------------------------------

import os
from glob import glob
from pygame import Surface, SRCALPHA
from albow.resource import resource_path, get_image
from albow.image_array import get_image_array
from random_mapping import RandomMapping

character_image_size = (30, 40)
default_character_image = get_image("characters", "default.png")

rnd = RandomMapping(seed = "characters")

renamings = {
	'KATHARINE': 'QUEEN KATHARINE',
	'LADY  CAPULET': 'LADY CAPULET',
	'ALL': None,
}

class CharacterInfo(object):
	#  name   string
	#  image  Surface
	pass

def load_kit(layer, keys):
	#print "load_kit:", keys ###
	kit = {}
	for key in keys:
		if isinstance(key, str):
			keystr = key
		else:
			keystr = "_".join(key)
		namepat = "chr_%s_%s-*.png" % (layer, keystr)
		pathpat = resource_path("images", namepat)
		#print "pathpat:", repr(pathpat) ###
		paths = glob(pathpat)
		#print paths
		if paths:
			path = paths[0]
			filename = os.path.basename(path)
			tail = filename.split("-")[1]
			shape = int(os.path.splitext(tail)[0])
			images = get_image_array(filename, shape, border = 2)
			kit[key] = images
	#print "kit =", kit ###
	return kit

ages = ('young', 'old')
sexes = ('male', 'female')
ages_and_sexes = [(age, sex) for age in ages for sex in sexes]

bodies = load_kit('body', sexes)
faces = load_kit('face', ages_and_sexes)
hair = load_kit('hair', ages_and_sexes)
crowns = load_kit('crown', sexes)

def make_character_image(name, flags):
	#print "make_character_image:", name, flags ###
	age = None
	sex = None
	crowned = False
	for flag in flags:
		if flag in ages:
			age = flag
		elif flag in sexes:
			sex = flag
		elif flag in ('king', 'queen'):
			crowned = True
	age_and_sex = (age, sex)
	buf = Surface(character_image_size, SRCALPHA)
	def compose(kit, layer, key):
		images = kit.get(key)
		if images:
			#print "composing from", images ###
			image = images[rnd.range((layer, name), len(images))]
			buf.blit(image, (0, 0))
	compose(bodies, 'body', sex)
	compose(faces, 'face', age_and_sex)
	compose(hair, 'hair', age_and_sex)
	if crowned:
		compose(crowns, 'crown', sex)
	return buf

def load_character_file(info_dict, path):
	f = open(path, "rU")
	for line in f:
		if not line.startswith("#"):
			if ":" in line:
				line = line.strip()
				#print line ###
				name, desc = line.split(":")
				flags = desc.split(",")
				image = make_character_image(name, flags)
				if image:
					#print "Storing character info:", name, image ###
					info = CharacterInfo()
					info.name = name
					info.image = image
					info_dict[name] = info

def load_character_info():
	info = {}
	dir = resource_path("plays")
	for filename in os.listdir(dir):
		if filename.endswith(".chr"):
			path = os.path.join(dir, filename)
			load_character_file(info, path)
	return info

character_info = load_character_info()

def get_character_info(name):
	#print "Getting character info for", name ###
	#name = "HAMLET" ###
	name = renamings.get(name, name)
	if name:
		return character_info.get(name)

def character_known(name):
	return get_character_info(name) is not None

def get_character_image(name):
	info = get_character_info(name)
	if info:
		return info.image
	else:
		return default_character_image
