#---------------------------------------------------------------------------
#
#   HIIW - Help
#
#---------------------------------------------------------------------------

from albow.resource import get_text, get_image, get_font
from albow.image_array import get_image_array
from albow.widget import Widget
from albow.controls import Label, Button, Image
from albow.layout import Row, Column
from albow.dialogs import Dialog
from game import game

text_font = get_font(15, "Hanshand.ttf")
image_title_font = get_font(15, "Hanshand.ttf")
image_title_spacing = 5
inter_image_spacing = 20
image_text_spacing = 10
paragraph_spacing = 10
pointer_spacing = 10

#---------------------------------------------------------------------------

class HelpBox(Dialog):
	
	align = 'c'
	pointer_image = None
	pointer_corner = None

	def __init__(self, rsrc_name, condition = None):
		Dialog.__init__(self)
		self.condition = condition
		self.items = []
		data = get_text(rsrc_name).split("\n")
		text_lines = []
		def flush_text():
			if text_lines:
				self.add_text("\n".join(text_lines))
				del text_lines[:]
		for line in data:
			if line.startswith("@"):
				fields = line[1:].split(",")
				self.topleft = map(int, fields)
			elif line.startswith("!"):
				self.align = line[1:]
			elif line.startswith("*"):
				self.set_pointer(line[1:])
			elif line.startswith("["):
				flush_text()
				self.add_images(line[1:-1])
			elif line.startswith("{"):
				flush_text()
				self.add_image_with_text(line[1:])
			elif line:
				text_lines.append(line)
			else:
				flush_text()
		flush_text()
		contents = Column(self.items, align = self.align, spacing = paragraph_spacing)
		m = self.margin
		contents.topleft = (m, m)
		self.contents = contents
		self.add(contents)
		self.position_pointer()
		self.shrink_wrap()
	
	def set_pointer(self, data):
		corner, rsrc = data.split(",")
		self.pointer_image = self.load_image(rsrc)
		self.pointer_corner = corner
	
	def position_pointer(self):
		img = self.pointer_image
		cont = self.contents
		if img:
			corner = self.pointer_corner
			if corner == 'bottomright':
				img.top = cont.bottom + pointer_spacing
				img.right = cont.right
			elif corner == 'topright':
				img.top = cont.top
				img.left = cont.right + pointer_spacing
			self.add(img)

	def add_text(self, text):
		self.items.append(Label(text, font = text_font))
	
	def add_images(self, data):
		boxes = []
		for item in data.split(","):
			title, rsrc = item.split("=")
			image_view = self.load_image(rsrc)
			title_view = Label(title, font = image_title_font)
			box = Column([image_view, title_view], spacing = image_title_spacing)
			boxes.append(box)
		self.items.append(Row(boxes, spacing = inter_image_spacing))
	
	def add_image_with_text(self, data):
		rsrc, text = data.split("}")
		image_view = self.load_image(rsrc)
		text_view = Label(text, font = text_font)
		box = Row([image_view, text_view], spacing = image_text_spacing)
		self.items.append(box)

	def load_image(self, rsrc):
		if "[" in rsrc:
			rsrc, index = rsrc[:-1].split("[")
			index, size = index.split("/")
			if "+" in size:
				size, border = size.split("+")
			else:
				border = "0"
			image = get_image_array(rsrc, int(size), border = int(border))[int(index)]
		else:
			image = get_image(rsrc)
		return Image(image)
	
	def get_visible(self):
		cond = self.condition
		return game.help_on and (not cond or cond())

#---------------------------------------------------------------------------

class HelpButton(Button):

	def __init__(self):
		Button.__init__(self, "XXXX Help (F1)")
	
	def get_text(self):
		if game.help_on:
			text = "Hide "
		else:
			text = "Show "
		return text + " Help (F1)"

	def action(self):
		game.toggle_help()
