#---------------------------------------------------------------------------
#
#   HIIW - Page drawing
#
#---------------------------------------------------------------------------

from textwrap import TextWrapper
from pygame import Color, Surface, SRCALPHA, BLEND_RGBA_MULT
from albow.resource import get_font, get_image

font_name = "Hanshand.ttf"
paper_image = get_image("paper.png")
paper_torn_mask = get_image("paper_torn_mask.png")
title_font = get_font(15, font_name)
page_font = get_font(13, font_name)
ink_color = Color("black")

title_top_margin = 20
max_chars_per_title_line = 20

top_margin = 10
left_margin = 10
indentation = 20
max_chars_per_line = 50

wrapper = TextWrapper(width = max_chars_per_line)
page_buffer = Surface(paper_image.get_size(), SRCALPHA)

def draw_sample(surf, sample, page_rect):
	output_lines = []
	def put(align, text):
		output_lines.append((align, text))
	def wrap(align, text):
		for line in wrapper.wrap(text):
			output_lines.append((align, line))
	def left(text):
		put('l', text)
	def indent(text):
		put('i', text)
	def center(text):
		put('c', text)
	def skip():
		output_lines.append(None)
	if sample.speech_no == 0:
		if sample.scene_idx == 1:
			center("ACT %s" % sample.act_idx)
		center("SCENE %s" % sample.scene_idx)
		if sample.scene_title:
			center(sample.scene_title)
		skip()
		if sample.scene_directions:
			for line in sample.scene_directions:
				wrap('l', line)
			skip()
	for character, lines in sample.speeches:
		left(character.upper())
		for line in lines:
			if isinstance(line, tuple):
				wrap('i', "(%s)" % line[1])
			else:
				indent(line)
		skip()
	line_height = page_font.get_linesize()
	xl = page_rect.left + left_margin
	xi = xl + indentation
	xc = page_rect.centerx
	y = page_rect.top + top_margin
	ymax = page_rect.bottom - line_height
	for line in output_lines:
		if y > ymax:
			break
		if line:
			align, text = line
			buf = page_font.render(text, True, ink_color)
			r = buf.get_rect()
			if align == 'l':
				r.topleft = (xl, y)
			elif align == 'i':
				r.topleft = (xi, y)
			else:
				r.midtop = (xc, y)
			surf.blit(buf, r)
		y += line_height

def draw_title(surf, play, page_rect):
	wrapper = TextWrapper(width = max_chars_per_title_line)
	y = page_rect.top + title_top_margin
	lines = wrapper.wrap(play.title)
	for line in lines:
		buf = title_font.render(line, True, ink_color)
		r = buf.get_rect()
		r.midtop = page_rect.centerx, y
		surf.blit(buf, r)
		y += title_font.get_linesize()

def render_page(surf, page, topleft):
	r = paper_image.get_rect()
	r.topleft = topleft
	surf.blit(paper_image, r)
	if page.is_title:
		draw_title(surf, page.play, r)
	else:
		sample = page.sample
		if sample:
			draw_sample(surf, sample, r)

def render_mask(surf, mask):
	surf.blit(mask, (0, 0), None, BLEND_RGBA_MULT)

def draw_page(surf, page, topleft):
	if page.destroyed:
		buf = page_buffer
		render_page(buf, page, (0, 0))
		render_mask(buf, paper_torn_mask)
		surf.blit(buf, topleft)
	else:
		render_page(surf, page, topleft)
