#---------------------------------------------------------------------------
#
#   HIIW - Plays
#
#---------------------------------------------------------------------------

import os
from zipfile import ZipFile
from cPickle import loads
from albow.resource import resource_path

max_sample_lines = 20

#---------------------------------------------------------------------------

class Play(object):
	#  name   string
	#  title  string
	
	def __init__(self, name, title):
		self.name = name
		self.title = title
		self.samples = []
	
	def add_sample(self, sample):
		self.samples.append(sample)
		sample.play = self
		if sample.pre_sample:
			sample.pre_sample.play = self

#---------------------------------------------------------------------------

class Sample(object):
	#  play         Play
	#  pre_sample   Sample
	#  character    string   Only in presamples
	
	play = None
	act_idx = None
	scene_idx = None
	speech_no = None
	scene_title = None
	scene_directions = None
	pre_sample = None
	
	def __init__(self, act_idx, scene_idx, scene_title, scene_dirs, speech_no):
		self.act_idx = act_idx
		self.scene_idx = scene_idx
		self.scene_title = scene_title
		self.scene_directions = scene_dirs
		self.line_count = 0
		self.speech_no = speech_no
		self.speeches = []
	
	def add_speech(self, character, items):
		n = min(len(items), max_sample_lines - self.line_count)
		self.speeches.append((character, items[:n]))
		self.line_count += n
	
	def __repr__(self):
		play = self.play
		if play:
			play_name = play.name
		else:
			play_name = "?"
		return "Sample<%s,act%s,scene%s,#%s>" % (self.play.name,
			self.act_idx, self.scene_idx, self.speech_no)

#---------------------------------------------------------------------------

def parse_key(key):
	parts = key.split("-")
	return int(parts[1]), int(parts[3])

def sample_act(rnd, zip, key):
	act_idx, scene_idx = parse_key(key)
	pcl = zip.read(key)
	data = loads(pcl)
	scene_title, scene_directions, items = data
	#print "Scene title:", scene_title
	n = len(items)
	i0 = rnd.range("first_item", n - 1)
	pre_sample = None
	if i0 < n - 1 and len(items[i0][1]) <= 4:
		#print "Getting presample from act", act_idx, "scene", scene_idx, "speech", i0 ###
		pre_sample = Sample(act_idx, scene_idx, None, None, i0)
		item = items[i0]
		pre_sample.character = item[0]
		pre_sample.add_speech(*item)
		i0 += 1
	i = i0
	sample = Sample(act_idx, scene_idx, scene_title, scene_directions, i)
	while i < n and sample.line_count < max_sample_lines:
		character, lines = items[i]
		if i == i0 and len(lines) > 10 and rnd.chance("split_chance", 0.5):
			character = ""
			k = rnd.range("split_point", 2, len(lines) - 1)
			lines = lines[k:]
			pre_sample = None
		sample.add_speech(character, lines)
		i += 1
	sample.pre_sample = pre_sample
	return sample

def open_play_file(play_name):
	path = resource_path("plays", play_name + ".zip")
	return ZipFile(path, "r")

def get_play_title(f):
	return f.read("title")

def get_play_index(f):
	result = []
	for key in f.namelist():
		if key.startswith("act-"):
			result.append(key)
	return result

def get_play_names():
	result = []
	plays_dir = resource_path("plays")
	for filename in os.listdir(plays_dir):
		if filename.endswith(".zip"):
			result.append(filename[:-4])
	return result

def get_play_samples(rnd, num_plays, num_samples_per_play):
	result = []
	play_names = get_play_names()
	for i, play_name in enumerate(rnd.choose("plays", num_plays, play_names)):
		f = open_play_file(play_name)
		title = get_play_title(f)
		play = Play(play_name, title)
		keys = get_play_index(f)
		prnd = rnd.fork("acts")
		sample_keys = list(prnd.choose(i, num_samples_per_play, keys))
		sample_keys.sort()
		for key in sample_keys:
			srnd = prnd.fork(key)
			sample = sample_act(srnd, f, key)
			play.add_sample(sample)
		result.append(play)
	return result
