#---------------------------------------------------------------------------
#
#   HIIW - Random mapping
#
#---------------------------------------------------------------------------

from __future__ import division
from hashlib import md5
from struct import unpack
from time import time

class RandomMapping(object):

	def __init__(self, state = None, seed = None):
		if not state:
			state = md5()
			if seed is None:
				seed = time()
				print "Seed:", seed
			state.update(str(seed))
		self.state = state
	
	def fork_state(self, key):
		s = self.state.copy()
		s.update(str(hash(key)))
		return s

	def int(self, key):
		s = self.fork_state(key)
		d = s.digest()[:4]
		return unpack("!L", d)[0]
	
	def float(self, key):
		i = self.int(key)
		return i / 0xffffffff
	
	def range(self, key, lo, hi = None):
		if hi is None:
			hi = lo
			lo = 0
		if hi > lo:
			i = self.int(key)
			return lo + i % (hi - lo)
		else:
			return lo
	
	def choice(self, key, items):
		return items[self.range(key, len(items))]
	
	def chance(self, key, p):
		return self.float(key) <= p

	def fork(self, *key):
		return RandomMapping(self.fork_state(key))

	def permuted_range(self, key, *args):
		nums = range(*args)
		seq = self.fork(key)
		i = 0
		while nums:
			j = seq.range(i, len(nums))
			yield nums.pop(j)
			i += 1

	def choose(self, key, count, items):
		return self.choose_and_pop(key, count, list(items))

	def choose_and_pop(self, key, count, pool):
		seq = self.fork(key)
		i = 0
		while pool and i < count:
			j = seq.range(i, len(pool))
			yield pool.pop(j)
			i += 1

	def permute(self, key, items):
		return self.choose(key, len(items), items)
