#---------------------------------------------------------------------------
#
#   HIIW - Game
#
#---------------------------------------------------------------------------

from __future__ import division
import os
from albow.resource import resource_path, get_image

class Carpet(object):
	#  image   Surface
	#  size    (width, height) in cells

	def __init__(self, filename):
		#print "Loading carpet image", filename ###
		image = get_image(filename)
		w, h = image.get_size()
		self.image = image
		self.width = w / 48
		self.height = h / 48

def get_carpets():
	result = []
	dir = resource_path("images", "carpets")
	for name in os.listdir(dir):
		if not name.startswith("."):
			filename = os.path.join("carpets", name)
			result.append(Carpet(filename))
	return result

carpets = get_carpets()

def pick_carpet(rnd, room_bounds):
	l, t, r, b = room_bounds
	maxw = r - l
	maxh = b - t
	#print "pick_carpet: for room size", (maxw, maxh) ###
	for carpet in rnd.permute("carpet_order", carpets):
		if carpet.width <= maxw and carpet.height <= maxh:
			i = carpets.index(carpet)
			#print "..carpet", i, "has size", (carpet.width, carpet.height) ###
			return i

class RoomStyle(object):
	
	def decorate_room(self, room):
		rnd = room.rnd
		if rnd.chance("carpet", self.carpet_chance):
			room.carpet_no = pick_carpet(rnd, room.bounds)

class LivingRoomStyle(RoomStyle):

	carpet_chance = 0.75

living_room = LivingRoomStyle()

class FloorStyle(object):
	
	def get_room_style(self, rnd):
		mand = self.mandatory_rooms
		if mand:
			return rnd.choice("style", mand)
		else:
			return rnd.choice("style", self.optional_rooms)

class GroundFloorStyle(FloorStyle):

	optional_rooms = [living_room]

	def __init__(self):
		self.mandatory_rooms = [living_room]

UpperFloorStyle = GroundFloorStyle

def create_floor_styles():
	return [GroundFloorStyle(),
		UpperFloorStyle(), UpperFloorStyle(), UpperFloorStyle()]
