#-------------------------------------------------------------------------
#
#   Suck in an HTML file containing a play and pick it apart.
#
#-------------------------------------------------------------------------

import os, re, sys
from cPickle import dumps
from zipfile import ZipFile, ZIP_DEFLATED

outdir = "Game/Resources/plays"

#-------------------------------------------------------------------------

title_pat = re.compile(r'<tr><td class="play" align="center">(.*)')
act_pat = re.compile(r"<H3>ACT (\w+)")
scene_pat = re.compile(r"<h3>SCENE (\w+)\. ([^<]*)")
speech_pat = re.compile(r"<A NAME=speech[^>]*><b>([^<]*)")
text_pat = re.compile(r"<A NAME=[^>]*>([^<]*)")
sd_pat = re.compile(r"<i>([^<]*)")

#-------------------------------------------------------------------------

class Scanner(object):

	linetype = ''
	title = ''
	act_no = ''
	scene_no = ''
	character = ''
	text = ''

	def __init__(self, path):
		f = open(path, "rU")
		self.f = f
		self.lineno = 0
		while 1:
			line = f.readline()
			if not line:
				return
			self.lineno += 1
			m = title_pat.match(line)
			if m:
				self.linetype = 'title'
				self.title = m.group(1)
				break
	
	def advance(self):
		self.next_line()
		while self.linetype == 'other':
			self.next_line()
		print self.linetype, self.text
	
	def next_line(self):
		line = self.f.readline()
		if not line:
			self.linetype = ''
			return
		self.lineno += 1
		line = line.strip()
		self.line = line
		self.text = ''
		m = act_pat.match(line)
		if m:
			self.linetype = 'act'
			self.act_no = m.group(1)
			self.scene_no = ''
			self.character = ''
			return
		m = scene_pat.match(line)
		if m:
			self.linetype = 'scene'
			self.scene_no = m.group(1)
			self.character = ''
			self.text = m.group(2)
			return
		m = speech_pat.match(line)
		if m:
			self.linetype = 'speech'
			self.character = m.group(1)
			return
		m = text_pat.match(line)
		if m:
			self.linetype = 'text'
			self.text = m.group(1)
			return
		m = sd_pat.search(line)
		if m:
			self.linetype = 'sd'
			self.text = m.group(1)
			return
		self.linetype = 'other'
	
	def eof(self):
		return self.linetype == ''
	
	def error(self, mess):
		raise Exception("Line %d: %s" % (self.lineno, mess))

#-------------------------------------------------------------------------

def open_zipfile(inpath):
	inname = os.path.basename(inpath)
	outname = os.path.splitext(inname)[0] + ".zip"
	zippath = os.path.join(outdir, outname)
	print "Writing", zippath
	return ZipFile(zippath, "w", ZIP_DEFLATED)

def test_scanner(s):
	print s.linetype, repr(s.title)
	while 1:
		if not s.linetype:
			break
		if s.linetype != 'other':
			print s.linetype, s.act_no, s.scene_no, repr(s.character), repr(s.text)
		s.next_line()

def read_speech(s):
	lines = []
	while 1:
		if s.linetype == 'text':
			lines.append(s.text)
		elif s.linetype == 'sd':
			lines.append(('d', s.text))
		else:
			break
		s.advance()
	return lines

def read_scene(s, act_idx, scene_idx, zip):
	scene_title = s.text
	s.advance()
	scene_directions = []
	while s.linetype in ('sd', 'text'):
		scene_directions.append(s.text)
		s.advance()
	items = []
	while s.linetype == 'speech':
		character = s.character
		s.advance()
		lines = read_speech(s)
		items.append((character, lines))
	pcl = dumps((scene_title, scene_directions, items), 2)
	key = "act-%02d-scene-%02d" % (act_idx, scene_idx)
	print "%s: %d items" % (key, len(items))
	zip.writestr(key, pcl)

def read_act(s, act_idx, zip):
	s.advance()
	scene_idx = 0
	while not s.eof() and s.linetype != 'scene': # skip prologue
		s.advance()
	while s.linetype == 'scene':
		scene_idx += 1
		read_scene(s, act_idx, scene_idx, zip)

def read_file(path):
	s = Scanner(path)
	if s.linetype != 'title':
		s.error("Title not found")
	zip = open_zipfile(path)
	zip.writestr("title", s.title)
	s.advance()
	act_idx = 0
	while s.linetype == 'act':
		act_idx += 1
		read_act(s, act_idx, zip)
	if s.linetype:
		s.error("Stuck at line type %r" % s.linetype)
	zip.close()

def main():
	inpath = sys.argv[1]
	read_file(inpath)

main()
