#-------------------------------------------------------------------------
#
#   Test extraction of a random play fragment.
#
#-------------------------------------------------------------------------

import sys
from zipfile import ZipFile
from cPickle import loads
import random

#-------------------------------------------------------------------------

class Sample(object):
	act_idx = None
	scene_idx = None
	scene_title = None
	scene_directions = None
	
	def __init__(self, act_idx, scene_idx, scene_title, scene_dirs, speech_no):
		if speech_no == 0:
			self.scene_idx = scene_idx
			self.scene_title = scene_title
			self.scene_directions = scene_dirs
			if scene_idx == 1:
				self.act_idx = act_idx
		self.line_count = 0
		self.speeches = []
	
	def add_speech(self, character, items):
		n = min(len(items), max_sample_lines - self.line_count)
		self.speeches.append((character, items[:n]))
		self.line_count += n

#-------------------------------------------------------------------------

def print_speech(speech):
	character, lines = speech
	if character:
		print character.upper()
	for line in lines:
		if isinstance(line, str):
			print "   ", line
		else:
			print "    [%s]" % line[1]

def print_sample(sample):
	if sample.act_idx:
		print "Act", sample.act_idx
	if sample.scene_idx:
		print "Scene %s:" % sample.scene_idx, sample.scene_title
		for line in sample.scene_directions:
			print "[%s]" % line
	for speech in sample.speeches:
		print_speech(speech)
		
def parse_key(key):
	parts = key.split("/")
	return int(parts[1]), int(parts[2])

max_sample_lines = 20

def sample_act(zip, key):
	act_idx, scene_idx = parse_key(key)
	pcl = zip.read(key)
	data = loads(pcl)
	scene_title, scene_directions, items = data
	#print "Scene title:", scene_title
	n = len(items)
	i0 = random.randrange(n - 1)
	i = i0
	sample = Sample(act_idx, scene_idx, scene_title, scene_directions, i)
	while i < n and sample.line_count < max_sample_lines:
		character, lines = items[i]
		if i == i0 and len(lines) > 10 and random.random() > 0.5:
			character = ""
			k = random.randrange(2, len(lines) - 1)
		sample.add_speech(character, lines)
		i += 1
	return sample

def main():
	path = sys.argv[1]
	zip = ZipFile(path, "r")
	keys = zip.namelist()
	keys.pop(0)
	#print "Zipfile contains:", keys
	title = zip.read("title")
	print "Play title:", repr(title)
	key = random.choice(keys)
	print "Sampling", key
	sample = sample_act(zip, key)
	print "--------------------------------------"
	print_sample(sample)

main()
